@@ -1,4 +1,5 @@
 import os
+import platform
 import re
 import socket
 import stat
@@ -130,11 +131,7 @@ def require(features):
         sys.exit(1)
 
 
-def matchoutput(cmd, regexp, ignorestatus=False):
-    """Return the match object if cmd executes successfully and its output
-    is matched by the supplied regular expression.
-    """
-
+def environ():
     # Tests on Windows have to fake USERPROFILE to point to the test area so
     # that `~` is properly expanded on py3.8+.  However, some tools like black
     # make calls that need the real USERPROFILE in order to run `foo --version`.
@@ -142,6 +139,14 @@ def matchoutput(cmd, regexp, ignorestatus=False):
     if os.name == 'nt':
         env = os.environ.copy()
         env['USERPROFILE'] = env['REALUSERPROFILE']
+    return env
+
+
+def matchoutput(cmd, regexp: bytes, ignorestatus: bool = False):
+    """Return the match object if cmd executes successfully and its output
+    is matched by the supplied regular expression.
+    """
+    env = environ()
 
     r = re.compile(regexp)
     p = subprocess.Popen(
@@ -171,7 +176,7 @@ def has_bzr():
         import breezy.revision
         import breezy.revisionspec
 
-        breezy.revisionspec.RevisionSpec
+        del breezy.revisionspec
         if breezy.__doc__ is None or breezy.version_info[:2] < (3, 1):
             return False
     except (AttributeError, ImportError):
@@ -248,7 +253,7 @@ def has_eol_in_paths():
         os.close(fd)
         os.remove(path)
         return True
-    except (IOError, OSError):
+    except OSError:
         return False
 
 
@@ -265,7 +270,7 @@ def has_executablebit():
             exec_flags_cannot_flip = (os.stat(fn).st_mode & 0o777) == m
         finally:
             os.unlink(fn)
-    except (IOError, OSError):
+    except OSError:
         # we don't care, the user probably won't be able to commit anyway
         return False
     return not (new_file_has_exec or exec_flags_cannot_flip)
@@ -336,7 +341,7 @@ def has_lsprof():
     try:
         import _lsprof
 
-        _lsprof.Profiler  # silence unused import warning
+        del _lsprof  # silence unused import warning
         return True
     except ImportError:
         return False
@@ -345,8 +350,8 @@ def has_lsprof():
 def _gethgversion():
     m = matchoutput('hg --version --quiet 2>&1', br'(\d+)\.(\d+)')
     if not m:
-        return (0, 0)
-    return (int(m.group(1)), int(m.group(2)))
+        return 0, 0
+    return int(m.group(1)), int(m.group(2))
 
 
 _hgversion = None
@@ -382,21 +387,23 @@ def has_rust():
 def has_hg08():
     if checks["hg09"][0]():
         return True
-    return matchoutput('hg help annotate 2>&1', '--date')
+    return matchoutput('hg help annotate 2>&1', b'--date')
 
 
 @check("hg07", "Mercurial >= 0.7")
 def has_hg07():
     if checks["hg08"][0]():
         return True
-    return matchoutput('hg --version --quiet 2>&1', 'Mercurial Distributed SCM')
+    return matchoutput(
+        'hg --version --quiet 2>&1', b'Mercurial Distributed SCM'
+    )
 
 
 @check("hg06", "Mercurial >= 0.6")
 def has_hg06():
     if checks["hg07"][0]():
         return True
-    return matchoutput('hg --version --quiet 2>&1', 'Mercurial version')
+    return matchoutput('hg --version --quiet 2>&1', b'Mercurial version')
 
 
 @check("gettext", "GNU Gettext (msgfmt)")
@@ -412,8 +419,8 @@ def has_git():
 def getgitversion():
     m = matchoutput('git --version 2>&1', br'git version (\d+)\.(\d+)')
     if not m:
-        return (0, 0)
-    return (int(m.group(1)), int(m.group(2)))
+        return 0, 0
+    return int(m.group(1)), int(m.group(2))
 
 
 @check("pygit2", "pygit2 Python library")
@@ -421,7 +428,7 @@ def has_pygit2():
     try:
         import pygit2
 
-        pygit2.Oid  # silence unused import
+        del pygit2  # silence unused import
         return True
     except ImportError:
         return False
@@ -450,7 +457,7 @@ def has_docutils():
     try:
         import docutils.core
 
-        docutils.core.publish_cmdline  # silence unused import
+        del docutils.core  # silence unused import
         return True
     except ImportError:
         return False
@@ -459,8 +466,8 @@ def has_docutils():
 def getsvnversion():
     m = matchoutput('svn --version --quiet 2>&1', br'^(\d+)\.(\d+)')
     if not m:
-        return (0, 0)
-    return (int(m.group(1)), int(m.group(2)))
+        return 0, 0
+    return int(m.group(1)), int(m.group(2))
 
 
 @checkvers("svn", "subversion client and admin tools >= %s", ('1.3', '1.5'))
@@ -609,14 +616,7 @@ def has_root():
 
 @check("pyflakes", "Pyflakes python linter")
 def has_pyflakes():
-    try:
-        import pyflakes
-
-        pyflakes.__version__
-    except ImportError:
-        return False
-    else:
-        return True
+    return matchoutput("pyflakes --version", br"^\d+\.\d+\.\d+\b", True)
 
 
 @check("pylint", "Pylint python linter")
@@ -624,12 +624,13 @@ def has_pylint():
     return matchoutput("pylint --help", br"[Uu]sage:[ ]+pylint", True)
 
 
-@check("clang-format", "clang-format C code formatter (>= 11)")
+@check("clang-format", "clang-format C code formatter (11 <= ... < 19)")
 def has_clang_format():
     m = matchoutput('clang-format --version', br'clang-format version (\d+)')
-    # style changed somewhere between 10.x and 11.x
+    # style changed somewhere between 10.x and 11.x and after 19.
     if m:
-        return int(m.group(1)) >= 11
+        major_version = int(m.group(1))
+        return 11 <= major_version < 19
     # Assist Googler contributors, they have a centrally-maintained version of
     # clang-format that is generally very fresh, but unlike most builds (both
     # official and unofficial), it does *not* include a version number.
@@ -648,7 +649,7 @@ def has_pygments():
     try:
         import pygments
 
-        pygments.highlight  # silence unused import warning
+        del pygments  # silence unused import warning
         return True
     except ImportError:
         return False
@@ -661,12 +662,14 @@ def getpygmentsversion():
         v = pygments.__version__
 
         parts = v.split(".")
-        return (int(parts[0]), int(parts[1]))
+        return int(parts[0]), int(parts[1])
     except ImportError:
-        return (0, 0)
+        return 0, 0
 
 
-@checkvers("pygments", "Pygments version >= %s", ('2.5', '2.11', '2.14'))
+@checkvers(
+    "pygments", "Pygments version >= %s", ('2.5', '2.11', '2.14', '2.19')
+)
 def has_pygments_range(v):
     major, minor = v.split('.')[0:2]
     return getpygmentsversion() >= (int(major), int(minor))
@@ -683,7 +686,7 @@ def has_ssl():
     try:
         import ssl
 
-        ssl.CERT_NONE
+        del ssl
         return True
     except ImportError:
         return False
@@ -712,11 +715,33 @@ def has_tls1_2():
     return b'tls1.2' in sslutil.supportedprotocols
 
 
+@check("tls1.3", "TLS 1.3 protocol support")
+def has_tls1_3():
+    from mercurial import sslutil
+
+    return b'tls1.3' in sslutil.supportedprotocols
+
+
 @check("windows", "Windows")
 def has_windows():
     return os.name == 'nt'
 
 
+@check("windows10", "Windows 10")
+def has_windows10():
+    # platform.win32_ver()[0] == '10' would be a better check, but doesn't
+    # return 11 on Windows 11 until Python 3.12(.7?).
+    if os.name != 'nt':
+        return False
+
+    vers = platform.win32_ver()[1].split('.')
+    if len(vers) == 3:
+        # Windows 11 build numbers start at 22000
+        return int(vers[2]) < 22000
+
+    return False
+
+
 @check("system-sh", "system() uses sh")
 def has_system_sh():
     return os.name != 'nt'
@@ -738,6 +763,13 @@ def has_setprocname():
         return False
 
 
+@check("gui", "whether a gui environment is available or not")
+def has_gui():
+    from mercurial.utils import procutil
+
+    return procutil.gui()
+
+
 @check("test-repo", "running tests from repository")
 def has_test_repo():
     t = os.environ["TESTDIR"]
@@ -755,7 +787,7 @@ def has_curses():
     try:
         import curses
 
-        curses.COLOR_BLUE
+        del curses
 
         # Windows doesn't have a `tic` executable, but the windows_curses
         # package is sufficient to run the tests without it.
@@ -799,12 +831,12 @@ def has_osx():
 def has_osxpackaging():
     try:
         return (
-            matchoutput('pkgbuild', br'Usage: pkgbuild ', ignorestatus=1)
+            matchoutput('pkgbuild', br'Usage: pkgbuild ', ignorestatus=True)
             and matchoutput(
-                'productbuild', br'Usage: productbuild ', ignorestatus=1
+                'productbuild', br'Usage: productbuild ', ignorestatus=True
             )
-            and matchoutput('lsbom', br'Usage: lsbom', ignorestatus=1)
-            and matchoutput('xar --help', br'Usage: xar', ignorestatus=1)
+            and matchoutput('lsbom', br'Usage: lsbom', ignorestatus=True)
+            and matchoutput('xar --help', br'Usage: xar', ignorestatus=True)
         )
     except ImportError:
         return False
@@ -917,7 +949,7 @@ def has_hypothesis():
     try:
         import hypothesis
 
-        hypothesis.given
+        del hypothesis
         return True
     except ImportError:
         return False
@@ -933,7 +965,7 @@ def has_zstd():
     try:
         import mercurial.zstd
 
-        mercurial.zstd.__version__
+        del mercurial.zstd
         return True
     except ImportError:
         return False
@@ -949,7 +981,7 @@ def has_ensurepip():
     try:
         import ensurepip
 
-        ensurepip.bootstrap
+        del ensurepip
         return True
     except ImportError:
         return False
@@ -978,7 +1010,7 @@ def has_fuzzywuzzy():
     try:
         import fuzzywuzzy
 
-        fuzzywuzzy.__version__
+        del fuzzywuzzy
         return True
     except ImportError:
         return False
@@ -1014,65 +1046,6 @@ def has_extraextensions():
     return 'HGTESTEXTRAEXTENSIONS' in os.environ
 
 
-def getrepofeatures():
-    """Obtain set of repository features in use.
-
-    HGREPOFEATURES can be used to define or remove features. It contains
-    a space-delimited list of feature strings. Strings beginning with ``-``
-    mean to remove.
-    """
-    # Default list provided by core.
-    features = {
-        'bundlerepo',
-        'revlogstore',
-        'fncache',
-    }
-
-    # Features that imply other features.
-    implies = {
-        'simplestore': ['-revlogstore', '-bundlerepo', '-fncache'],
-    }
-
-    for override in os.environ.get('HGREPOFEATURES', '').split(' '):
-        if not override:
-            continue
-
-        if override.startswith('-'):
-            if override[1:] in features:
-                features.remove(override[1:])
-        else:
-            features.add(override)
-
-            for imply in implies.get(override, []):
-                if imply.startswith('-'):
-                    if imply[1:] in features:
-                        features.remove(imply[1:])
-                else:
-                    features.add(imply)
-
-    return features
-
-
-@check('reporevlogstore', 'repository using the default revlog store')
-def has_reporevlogstore():
-    return 'revlogstore' in getrepofeatures()
-
-
-@check('reposimplestore', 'repository using simple storage extension')
-def has_reposimplestore():
-    return 'simplestore' in getrepofeatures()
-
-
-@check('repobundlerepo', 'whether we can open bundle files as repos')
-def has_repobundlerepo():
-    return 'bundlerepo' in getrepofeatures()
-
-
-@check('repofncache', 'repository has an fncache')
-def has_repofncache():
-    return 'fncache' in getrepofeatures()
-
-
 @check('dirstate-v2', 'using the v2 format of .hg/dirstate')
 def has_dirstate_v2():
     # Keep this logic in sync with `newreporequirements()` in `mercurial/localrepo.py`
@@ -1102,7 +1075,7 @@ def has_vcr():
     try:
         import vcr
 
-        vcr.VCR
+        del vcr
         return True
     except (ImportError, AttributeError):
         pass
@@ -1117,14 +1090,31 @@ def has_emacs():
     return matchoutput('emacs --version', b'GNU Emacs 2(4.4|4.5|5|6|7|8|9)')
 
 
-@check('black', 'the black formatter for python (>= 20.8b1)')
+@check(
+    'black', 'the black formatter for python (version set in pyproject.toml)'
+)
 def has_black():
-    blackcmd = 'black --version'
-    version_regex = b'black, (?:version )?([0-9a-b.]+)'
-    version = matchoutput(blackcmd, version_regex)
-    if not version:
+    env = environ()
+
+    # The top level #require statements are evaluated from $TESTTMP, and won't
+    # see the project level pyproject.toml unless executing from the project
+    # directory.
+    cwd = os.getcwd()
+    if 'RUNTESTDIR' in env:
+        cwd = os.path.realpath(f"{env['RUNTESTDIR']}/..")
+
+    try:
+        p = subprocess.Popen(
+            ['black', '--check', '-'],
+            stdin=subprocess.PIPE,
+            cwd=cwd,
+            env=env,
+        )
+    except FileNotFoundError:
         return False
-    return Version(_bytes2sys(version.group(1))) >= Version('20.8b1')
+
+    p.communicate(b'# test\n')
+    return p.returncode == 0
 
 
 @check('pytype', 'the pytype type checker')
@@ -1136,11 +1126,11 @@ def has_pytype():
     return Version(_bytes2sys(version.group(0))) >= Version('2019.10.17')
 
 
-@check("rustfmt", "rustfmt tool at version nightly-2021-11-02")
+@check("rustfmt", "rustfmt tool at version nightly-2024-07-16")
 def has_rustfmt():
     # We use Nightly's rustfmt due to current unstable config options.
     return matchoutput(
-        '`rustup which --toolchain nightly-2021-11-02 rustfmt` --version',
+        '`rustup which --toolchain nightly-2024-07-16 rustfmt` --version',
         b'rustfmt',
     )
 
@@ -1155,7 +1145,7 @@ def has_lzma():
     try:
         import _lzma
 
-        _lzma.FORMAT_XZ
+        del _lzma
         return True
     except ImportError:
         return False
